import { AbstractAstNode } from '@fern-api/generator-commons';
import { ModulePath, AttrPath } from './core/types.cjs';
import { Writer } from './core/Writer.cjs';

declare namespace Reference {
    interface Args {
        name: string;
        modulePath?: ModulePath;
        genericTypes?: AstNode[];
        alias?: string;
        attribute?: AttrPath;
    }
}
declare class Reference extends AstNode {
    readonly name: string;
    readonly modulePath: ModulePath;
    private readonly genericTypes;
    readonly alias: string | undefined;
    private readonly attribute;
    constructor({ name, modulePath, genericTypes, alias, attribute }: Reference.Args);
    write(writer: Writer): void;
    getFullyQualifiedModulePath(): string;
}

declare abstract class AstNode extends AbstractAstNode {
    protected references: Reference[];
    addReference(reference: Reference): void;
    inheritReferences(astNode: AstNode | undefined): void;
    getReferences(): Reference[];
    /**
     * Writes the node to a string.
     */
    toString(): string;
    /**
     * Writes the node to a string and formats it.
     */
    toStringFormatted(): Promise<string>;
}

export { AstNode as A, Reference as R };
