"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/Class.ts
var Class_exports = {};
__export(Class_exports, {
  Class: () => Class
});
module.exports = __toCommonJS(Class_exports);

// src/core/AstNode.ts
var import_generator_commons2 = require("@fern-api/generator-commons");

// src/core/Writer.ts
var import_generator_commons = require("@fern-api/generator-commons");
var Writer = class extends import_generator_commons.AbstractWriter {
  toString() {
    return this.buffer;
  }
  async toStringFormatted() {
    const { default: init, format } = await import("@wasm-fmt/ruff_fmt");
    await init();
    return format(this.buffer);
  }
};

// src/core/AstNode.ts
var AstNode = class extends import_generator_commons2.AbstractAstNode {
  references = [];
  addReference(reference) {
    this.references.push(reference);
  }
  inheritReferences(astNode) {
    if (astNode === void 0) {
      return;
    }
    astNode.references.forEach((reference) => {
      if (!this.references.includes(reference)) {
        this.addReference(reference);
      }
    });
  }
  getReferences() {
    return this.references;
  }
  /**
   * Writes the node to a string.
   */
  toString() {
    const writer = new Writer();
    this.write(writer);
    return writer.toString();
  }
  /**
   * Writes the node to a string and formats it.
   */
  async toStringFormatted() {
    const writer = new Writer();
    this.write(writer);
    return await writer.toStringFormatted();
  }
};

// src/Class.ts
var Class = class extends AstNode {
  name;
  extends_;
  decorators;
  fields = [];
  docs;
  statements = [];
  constructor({ docs, name, extends_, decorators }) {
    super();
    this.name = name;
    this.extends_ = extends_ ?? [];
    this.decorators = decorators ?? [];
    this.docs = docs;
    this.extends_.forEach((parentClassReference) => {
      this.addReference(parentClassReference);
    });
  }
  write(writer) {
    this.decorators.forEach((decorator) => {
      decorator.write(writer);
    });
    writer.write(`class ${this.name}`);
    if (this.extends_.length > 0) {
      writer.write("(");
      this.extends_.forEach((parentClassReference, index) => {
        if (index > 0) {
          writer.write(", ");
        }
        parentClassReference.write(writer);
      });
      writer.write(")");
    }
    writer.write(":");
    writer.newLine();
    if (this.docs != null) {
      writer.write('"""');
      writer.write(this.docs);
      writer.write('"""');
    }
    writer.writeNewLineIfLastLineNot();
    writer.indent();
    this.fields.forEach((field) => {
      field.write(writer);
      writer.writeNewLineIfLastLineNot();
    });
    writer.dedent();
    writer.indent();
    if (this.statements.length) {
      this.writeStatements({ writer });
    } else {
      writer.write("pass");
    }
    writer.dedent();
  }
  add(statement) {
    this.statements.push(statement);
    this.inheritReferences(statement);
  }
  writeStatements({ writer }) {
    this.statements.forEach((statement, index) => {
      statement.write(writer);
      writer.writeNewLineIfLastLineNot();
    });
  }
  addField(field) {
    this.add(field);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Class
});
