"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/PythonFile.ts
var PythonFile_exports = {};
__export(PythonFile_exports, {
  PythonFile: () => PythonFile
});
module.exports = __toCommonJS(PythonFile_exports);

// src/core/AstNode.ts
var import_generator_commons2 = require("@fern-api/generator-commons");

// src/core/Writer.ts
var import_generator_commons = require("@fern-api/generator-commons");
var Writer = class extends import_generator_commons.AbstractWriter {
  toString() {
    return this.buffer;
  }
  async toStringFormatted() {
    const { default: init, format } = await import("@wasm-fmt/ruff_fmt");
    await init();
    return format(this.buffer);
  }
};

// src/core/AstNode.ts
var AstNode = class extends import_generator_commons2.AbstractAstNode {
  references = [];
  addReference(reference) {
    this.references.push(reference);
  }
  inheritReferences(astNode) {
    if (astNode === void 0) {
      return;
    }
    astNode.references.forEach((reference) => {
      if (!this.references.includes(reference)) {
        this.addReference(reference);
      }
    });
  }
  getReferences() {
    return this.references;
  }
  /**
   * Writes the node to a string.
   */
  toString() {
    const writer = new Writer();
    this.write(writer);
    return writer.toString();
  }
  /**
   * Writes the node to a string and formats it.
   */
  async toStringFormatted() {
    const writer = new Writer();
    this.write(writer);
    return await writer.toStringFormatted();
  }
};

// src/PythonFile.ts
var PythonFile = class extends AstNode {
  moduleName;
  path;
  name;
  statements = [];
  constructor({ moduleName, path, name }) {
    super();
    this.moduleName = moduleName;
    this.path = path;
    this.name = name;
  }
  addStatement(statement) {
    this.statements.push(statement);
    this.inheritReferences(statement);
  }
  write(writer) {
    this.writeImports(writer);
    this.statements.forEach((statement, idx) => {
      statement.write(writer);
      writer.newLine();
      if (idx < this.statements.length - 1) {
        writer.newLine();
      }
    });
  }
  /*******************************
   * Helper Methods
   *******************************/
  getImportName(reference) {
    const name = reference.name;
    const alias = reference.alias;
    return `${name}${alias ? ` as ${alias}` : ""}`;
  }
  writeImports(writer) {
    const uniqueReferences = /* @__PURE__ */ new Map();
    for (const reference of this.references) {
      const fullyQualifiedPath = reference.getFullyQualifiedModulePath();
      const existingRefs = uniqueReferences.get(fullyQualifiedPath);
      const referenceName = reference.name;
      if (existingRefs) {
        if (!existingRefs.referenceNames.has(referenceName)) {
          existingRefs.references.push(reference);
          existingRefs.referenceNames.add(referenceName);
        }
      } else {
        uniqueReferences.set(fullyQualifiedPath, {
          modulePath: reference.modulePath,
          references: [reference],
          referenceNames: /* @__PURE__ */ new Set([referenceName])
        });
      }
    }
    for (const [fullyQualifiedPath, { modulePath, references }] of uniqueReferences) {
      const refModulePath = modulePath;
      if (refModulePath[0] === this.moduleName) {
        let commonPrefixLength = 0;
        while (commonPrefixLength < this.path.length && commonPrefixLength < refModulePath.length && this.path[commonPrefixLength] === refModulePath[commonPrefixLength]) {
          commonPrefixLength++;
        }
        const levelsUp = this.path.length - commonPrefixLength;
        let relativePath = levelsUp > 0 ? ".".repeat(levelsUp) : ".";
        relativePath += refModulePath.slice(commonPrefixLength).join(".");
        writer.write(
          `from ${relativePath} import ${references.map((ref) => this.getImportName(ref)).join(", ")}`
        );
      } else {
        writer.write(
          `from ${fullyQualifiedPath} import ${references.map((ref) => this.getImportName(ref)).join(", ")}`
        );
      }
      writer.newLine();
    }
    if (Object.keys(uniqueReferences).length > 0) {
      writer.newLine();
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  PythonFile
});
