"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/Type.ts
var Type_exports = {};
__export(Type_exports, {
  Type: () => Type
});
module.exports = __toCommonJS(Type_exports);
var import_core_utils3 = require("@fern-api/core-utils");

// src/python.ts
var python_exports = {};
__export(python_exports, {
  Class: () => Class,
  ClassInstantiation: () => ClassInstantiation,
  CodeBlock: () => CodeBlock,
  Decorator: () => Decorator,
  Field: () => Field,
  Method: () => Method,
  MethodArgument: () => MethodArgument,
  MethodInvocation: () => MethodInvocation,
  Operator: () => Operator,
  Parameter: () => Parameter,
  PythonFile: () => PythonFile,
  Reference: () => Reference,
  Type: () => Type,
  TypeInstantiation: () => TypeInstantiation,
  class_: () => class_,
  codeBlock: () => codeBlock,
  decorator: () => decorator,
  field: () => field,
  file: () => file,
  instantiateClass: () => instantiateClass,
  invokeMethod: () => invokeMethod,
  method: () => method,
  methodArgument: () => methodArgument,
  operator: () => operator,
  parameter: () => parameter,
  reference: () => reference
});

// src/core/AstNode.ts
var import_generator_commons2 = require("@fern-api/generator-commons");

// src/core/Writer.ts
var import_generator_commons = require("@fern-api/generator-commons");
var Writer = class extends import_generator_commons.AbstractWriter {
  toString() {
    return this.buffer;
  }
  async toStringFormatted() {
    const { default: init, format } = await import("@wasm-fmt/ruff_fmt");
    await init();
    return format(this.buffer);
  }
};

// src/core/AstNode.ts
var AstNode = class extends import_generator_commons2.AbstractAstNode {
  references = [];
  addReference(reference2) {
    this.references.push(reference2);
  }
  inheritReferences(astNode) {
    if (astNode === void 0) {
      return;
    }
    astNode.references.forEach((reference2) => {
      if (!this.references.includes(reference2)) {
        this.addReference(reference2);
      }
    });
  }
  getReferences() {
    return this.references;
  }
  /**
   * Writes the node to a string.
   */
  toString() {
    const writer = new Writer();
    this.write(writer);
    return writer.toString();
  }
  /**
   * Writes the node to a string and formats it.
   */
  async toStringFormatted() {
    const writer = new Writer();
    this.write(writer);
    return await writer.toStringFormatted();
  }
};

// src/Class.ts
var Class = class extends AstNode {
  name;
  extends_;
  decorators;
  fields = [];
  docs;
  statements = [];
  constructor({ docs, name, extends_, decorators }) {
    super();
    this.name = name;
    this.extends_ = extends_ ?? [];
    this.decorators = decorators ?? [];
    this.docs = docs;
    this.extends_.forEach((parentClassReference) => {
      this.addReference(parentClassReference);
    });
  }
  write(writer) {
    this.decorators.forEach((decorator2) => {
      decorator2.write(writer);
    });
    writer.write(`class ${this.name}`);
    if (this.extends_.length > 0) {
      writer.write("(");
      this.extends_.forEach((parentClassReference, index) => {
        if (index > 0) {
          writer.write(", ");
        }
        parentClassReference.write(writer);
      });
      writer.write(")");
    }
    writer.write(":");
    writer.newLine();
    if (this.docs != null) {
      writer.write('"""');
      writer.write(this.docs);
      writer.write('"""');
    }
    writer.writeNewLineIfLastLineNot();
    writer.indent();
    this.fields.forEach((field2) => {
      field2.write(writer);
      writer.writeNewLineIfLastLineNot();
    });
    writer.dedent();
    writer.indent();
    if (this.statements.length) {
      this.writeStatements({ writer });
    } else {
      writer.write("pass");
    }
    writer.dedent();
  }
  add(statement) {
    this.statements.push(statement);
    this.inheritReferences(statement);
  }
  writeStatements({ writer }) {
    this.statements.forEach((statement, index) => {
      statement.write(writer);
      writer.writeNewLineIfLastLineNot();
    });
  }
  addField(field2) {
    this.add(field2);
  }
};

// src/Field.ts
var Field = class extends AstNode {
  name;
  type;
  initializer;
  docs;
  constructor({ name, type, initializer, docs }) {
    super();
    this.name = name;
    this.type = type;
    this.initializer = initializer;
    this.docs = docs;
    this.inheritReferences(this.type);
    this.inheritReferences(this.initializer);
  }
  write(writer) {
    writer.write(this.name);
    if (this.type !== void 0) {
      writer.write(": ");
      this.type.write(writer);
    }
    if (this.initializer !== void 0) {
      writer.write(` = ${this.initializer}`);
    }
    if (this.docs != null) {
      writer.newLine();
      writer.write('"""');
      writer.newLine();
      writer.write(this.docs);
      writer.newLine();
      writer.write('"""');
    }
  }
};

// src/Reference.ts
var Reference = class extends AstNode {
  name;
  modulePath;
  genericTypes;
  alias;
  attribute;
  constructor({ name, modulePath, genericTypes, alias, attribute }) {
    super();
    this.name = name;
    this.modulePath = modulePath ?? [];
    this.genericTypes = genericTypes ?? [];
    this.alias = alias;
    this.attribute = attribute ?? [];
    this.references.push(this);
    this.genericTypes.forEach((genericType) => {
      this.inheritReferences(genericType);
    });
  }
  write(writer) {
    writer.write(this.alias ?? this.name);
    if (this.genericTypes.length > 0) {
      writer.write("[");
      this.genericTypes.forEach((genericType, index) => {
        if (index > 0) {
          writer.write(", ");
        }
        genericType.write(writer);
      });
      writer.write("]");
    }
    if (this.attribute.length > 0) {
      writer.write(".");
      this.attribute.forEach((attr, index) => {
        if (index > 0) {
          writer.write(".");
        }
        writer.write(attr);
      });
    }
  }
  getFullyQualifiedModulePath() {
    return this.modulePath.join(".");
  }
};

// src/CodeBlock.ts
var import_generator_commons3 = require("@fern-api/generator-commons");
var CodeBlock = class extends AstNode {
  arg;
  constructor(arg) {
    super();
    this.arg = arg;
  }
  write(writer) {
    const commonCodeBlock = new import_generator_commons3.CodeBlock(this.arg);
    return commonCodeBlock.write(writer);
  }
};

// src/Method.ts
var Method = class extends AstNode {
  name;
  return;
  docstring;
  type;
  parameters;
  decorators;
  statements = [];
  static_;
  constructor({ static_, name, parameters, return_, docstring, type, decorators }) {
    super();
    this.name = name;
    this.parameters = parameters ?? [];
    this.return = return_;
    this.docstring = docstring;
    this.type = type;
    this.decorators = decorators ?? [];
    this.static_ = static_ ?? false;
  }
  addStatement(statement) {
    this.statements.push(statement);
    this.inheritReferences(statement);
  }
  write(writer) {
    if (this.static_) {
      this.decorators.push(
        python_exports.decorator({
          callable: python_exports.codeBlock("staticmethod")
        })
      );
    }
    this.decorators.forEach((decorator2) => {
      decorator2.write(writer);
    });
    if (this.type === 2 /* CLASS */) {
      python_exports.decorator({
        callable: new CodeBlock("classmethod")
      }).write(writer);
    } else if (this.type === 0 /* STATIC */) {
      python_exports.decorator({
        callable: new CodeBlock("staticmethod")
      }).write(writer);
    }
    writer.write(`def ${this.name}(`);
    if (this.type === 1 /* INSTANCE */) {
      writer.write("self");
      if (this.parameters.length > 0) {
        writer.write(", ");
      }
    } else if (this.type === 2 /* CLASS */) {
      writer.write("cls");
      if (this.parameters.length > 0) {
        writer.write(", ");
      }
    }
    this.parameters.forEach((param, index) => {
      param.write(writer);
      if (index < this.parameters.length - 1) {
        writer.write(", ");
      }
    });
    writer.write(")");
    if (this.return) {
      writer.write(" -> ");
      this.return.write(writer);
    }
    writer.write(":");
    writer.newLine();
    if (this.docstring) {
      writer.indent();
      writer.write('"""');
      writer.write(this.docstring);
      writer.write('"""');
      writer.newLine();
      writer.dedent();
    }
    if (this.statements.length) {
      writer.indent();
      this.statements.forEach((statement, index) => {
        statement.write(writer);
        if (index < this.statements.length - 1) {
          writer.newLine();
        }
      });
      writer.dedent();
    } else {
      writer.indent();
      writer.write("pass");
      writer.dedent();
    }
    writer.newLine();
  }
};

// src/BaseInvocation.ts
var BaseInvocation = class extends AstNode {
  reference;
  arguments;
  constructor({ reference: reference2, arguments_ }) {
    super();
    this.reference = reference2;
    this.arguments = arguments_;
    this.inheritReferences(reference2);
    this.arguments.forEach((arg) => {
      this.inheritReferences(arg);
    });
  }
  write(writer) {
    this.reference.write(writer);
    writer.write("(");
    this.arguments.forEach((arg, idx) => {
      arg.write(writer);
      if (idx < this.arguments.length - 1) {
        writer.write(", ");
      }
    });
    writer.write(")");
  }
};

// src/MethodInvocation.ts
var MethodInvocation = class extends BaseInvocation {
  constructor({ methodReference, ...args }) {
    super({ reference: methodReference, ...args });
  }
};

// src/ClassInstantiation.ts
var ClassInstantiation = class extends BaseInvocation {
  constructor({ classReference, ...args }) {
    super({ reference: classReference, ...args });
  }
};

// src/Parameter.ts
var Parameter = class extends AstNode {
  name;
  initializer;
  type;
  constructor({ name, type, initializer }) {
    super();
    this.name = name;
    this.type = type;
    this.initializer = initializer;
  }
  write(writer) {
    writer.write(this.name);
    writer.write(": ");
    this.type.write(writer);
    if (this.initializer !== void 0) {
      writer.write(" = ");
      this.initializer.write(writer);
    }
  }
};

// src/MethodArgument.ts
var MethodArgument = class extends AstNode {
  name;
  value;
  constructor({ name, value }) {
    super();
    this.name = name;
    this.value = value;
    this.inheritReferences(this.value);
  }
  write(writer) {
    if (this.name !== void 0) {
      writer.write(this.name);
      writer.write("=");
    }
    this.value.write(writer);
  }
};

// src/PythonFile.ts
var PythonFile = class extends AstNode {
  moduleName;
  path;
  name;
  statements = [];
  constructor({ moduleName, path, name }) {
    super();
    this.moduleName = moduleName;
    this.path = path;
    this.name = name;
  }
  addStatement(statement) {
    this.statements.push(statement);
    this.inheritReferences(statement);
  }
  write(writer) {
    this.writeImports(writer);
    this.statements.forEach((statement, idx) => {
      statement.write(writer);
      writer.newLine();
      if (idx < this.statements.length - 1) {
        writer.newLine();
      }
    });
  }
  /*******************************
   * Helper Methods
   *******************************/
  getImportName(reference2) {
    const name = reference2.name;
    const alias = reference2.alias;
    return `${name}${alias ? ` as ${alias}` : ""}`;
  }
  writeImports(writer) {
    const uniqueReferences = /* @__PURE__ */ new Map();
    for (const reference2 of this.references) {
      const fullyQualifiedPath = reference2.getFullyQualifiedModulePath();
      const existingRefs = uniqueReferences.get(fullyQualifiedPath);
      const referenceName = reference2.name;
      if (existingRefs) {
        if (!existingRefs.referenceNames.has(referenceName)) {
          existingRefs.references.push(reference2);
          existingRefs.referenceNames.add(referenceName);
        }
      } else {
        uniqueReferences.set(fullyQualifiedPath, {
          modulePath: reference2.modulePath,
          references: [reference2],
          referenceNames: /* @__PURE__ */ new Set([referenceName])
        });
      }
    }
    for (const [fullyQualifiedPath, { modulePath, references }] of uniqueReferences) {
      const refModulePath = modulePath;
      if (refModulePath[0] === this.moduleName) {
        let commonPrefixLength = 0;
        while (commonPrefixLength < this.path.length && commonPrefixLength < refModulePath.length && this.path[commonPrefixLength] === refModulePath[commonPrefixLength]) {
          commonPrefixLength++;
        }
        const levelsUp = this.path.length - commonPrefixLength;
        let relativePath = levelsUp > 0 ? ".".repeat(levelsUp) : ".";
        relativePath += refModulePath.slice(commonPrefixLength).join(".");
        writer.write(
          `from ${relativePath} import ${references.map((ref) => this.getImportName(ref)).join(", ")}`
        );
      } else {
        writer.write(
          `from ${fullyQualifiedPath} import ${references.map((ref) => this.getImportName(ref)).join(", ")}`
        );
      }
      writer.newLine();
    }
    if (Object.keys(uniqueReferences).length > 0) {
      writer.newLine();
    }
  }
};

// src/Decorator.ts
var Decorator = class extends AstNode {
  callable;
  constructor({ callable }) {
    super();
    this.callable = callable;
    this.inheritReferences(callable);
  }
  write(writer) {
    writer.write("@");
    this.callable.write(writer);
    writer.newLine();
  }
};

// src/Operator.ts
var import_core_utils = require("@fern-api/core-utils");

// src/OperatorType.ts
var OperatorType = {
  Or: "or",
  And: "and",
  LeftShift: "leftShift",
  RightShift: "rightShift"
};

// src/Operator.ts
var Operator = class extends AstNode {
  operator;
  lhs;
  rhs;
  constructor({ operator: operator2, lhs, rhs }) {
    super();
    this.operator = operator2;
    this.lhs = lhs;
    this.inheritReferences(lhs);
    this.rhs = rhs;
    this.inheritReferences(rhs);
  }
  getOperatorString() {
    switch (this.operator) {
      case OperatorType.Or:
        return "or";
      case OperatorType.And:
        return "and";
      case OperatorType.LeftShift:
        return "<<";
      case OperatorType.RightShift:
        return ">>";
      default:
        (0, import_core_utils.assertNever)(this.operator);
    }
  }
  write(writer) {
    writer.write(`${this.lhs.toString()} `);
    writer.write(this.getOperatorString());
    writer.write(` ${this.rhs.toString()}`);
  }
};

// src/TypeInstantiation.ts
var import_core_utils2 = require("@fern-api/core-utils");
var TypeInstantiation = class extends AstNode {
  constructor(internalType) {
    super();
    this.internalType = internalType;
  }
  static int(value) {
    return new this({ type: "int", value });
  }
  static float(value) {
    return new this({ type: "float", value });
  }
  static bool(value) {
    return new this({ type: "bool", value });
  }
  static str(value) {
    return new this({ type: "str", value });
  }
  static bytes(value) {
    return new this({ type: "bytes", value });
  }
  static list(values) {
    const list = new this({ type: "list", values });
    values.forEach((value) => list.inheritReferences(value));
    return list;
  }
  static set(values) {
    const set = new this({ type: "set", values });
    values.forEach((value) => set.inheritReferences(value));
    return set;
  }
  static tuple(values) {
    const tuple = new this({ type: "tuple", values });
    values.forEach((value) => tuple.inheritReferences(value));
    return tuple;
  }
  static dict(entries) {
    const dict = new this({ type: "dict", entries });
    entries.forEach((entry) => {
      dict.inheritReferences(entry.key);
      dict.inheritReferences(entry.value);
    });
    return dict;
  }
  static none() {
    return new this({ type: "none" });
  }
  static uuid(value) {
    const uuid = new this({ type: "uuid", value });
    uuid.addReference(new Reference({ name: "UUID", modulePath: ["uuid"] }));
    return uuid;
  }
  write(writer) {
    switch (this.internalType.type) {
      case "int":
        writer.write(this.internalType.value.toString());
        break;
      case "float":
        writer.write(this.internalType.value.toString());
        break;
      case "bool":
        if (this.internalType.value) {
          writer.write("True");
        } else {
          writer.write("False");
        }
        break;
      case "str":
        writer.write(`"${this.internalType.value}"`);
        break;
      case "bytes":
        writer.write(`b"${this.internalType.value}"`);
        break;
      case "list":
        writer.write("[");
        this.internalType.values.forEach((value, index) => {
          if (index > 0) {
            writer.write(", ");
          }
          value.write(writer);
        });
        writer.write("]");
        break;
      case "set":
        writer.write("{");
        this.internalType.values.forEach((value, index) => {
          if (index > 0) {
            writer.write(", ");
          }
          value.write(writer);
        });
        writer.write("}");
        break;
      case "tuple":
        writer.write("(");
        this.internalType.values.forEach((value, index) => {
          if (index > 0) {
            writer.write(", ");
          }
          value.write(writer);
        });
        writer.write(")");
        break;
      case "dict":
        writer.write("{");
        this.internalType.entries.forEach((entry, index) => {
          if (index > 0) {
            writer.write(", ");
          }
          entry.key.write(writer);
          writer.write(": ");
          entry.value.write(writer);
        });
        writer.write("}");
        break;
      case "none":
        writer.write("None");
        break;
      case "uuid":
        writer.write(`UUID("${this.internalType.value}")`);
        break;
      default:
        (0, import_core_utils2.assertNever)(this.internalType);
    }
  }
};

// src/python.ts
function file(args) {
  return new PythonFile(args);
}
function class_(args) {
  return new Class(args);
}
function instantiateClass(args) {
  return new ClassInstantiation(args);
}
function decorator(args) {
  return new Decorator(args);
}
function reference(args) {
  return new Reference(args);
}
function field(args) {
  return new Field(args);
}
function codeBlock(args) {
  return new CodeBlock(args);
}
function method(args) {
  return new Method(args);
}
function invokeMethod(args) {
  return new MethodInvocation(args);
}
function parameter(args) {
  return new Parameter(args);
}
function methodArgument(args) {
  return new MethodArgument(args);
}
function operator(args) {
  return new Operator(args);
}

// src/Type.ts
var Type = class extends AstNode {
  internalType;
  constructor(internalType) {
    super();
    this.internalType = internalType;
  }
  static int() {
    return new this({ type: "int" });
  }
  static float() {
    return new this({ type: "float" });
  }
  static bool() {
    return new this({ type: "bool" });
  }
  static str() {
    return new this({ type: "str" });
  }
  static bytes() {
    return new this({ type: "bytes" });
  }
  static list(value) {
    const listType = new this({ type: "list", value });
    listType.addReference(python_exports.reference({ name: "List", modulePath: ["typing"] }));
    listType.inheritReferences(value);
    return listType;
  }
  static set(value) {
    const setType = new this({ type: "set", value });
    setType.addReference(python_exports.reference({ name: "Set", modulePath: ["typing"] }));
    setType.inheritReferences(value);
    return setType;
  }
  static tuple(values) {
    const tupleType = new this({ type: "tuple", values });
    tupleType.addReference(python_exports.reference({ name: "Tuple", modulePath: ["typing"] }));
    values.forEach((value) => tupleType.inheritReferences(value));
    return tupleType;
  }
  static dict(keyType, valueType) {
    const dictType = new this({ type: "dict", keyType, valueType });
    dictType.addReference(python_exports.reference({ name: "Dict", modulePath: ["typing"] }));
    dictType.inheritReferences(keyType);
    dictType.inheritReferences(valueType);
    return dictType;
  }
  static none() {
    return new this({ type: "none" });
  }
  static uuid() {
    const uuidType = new this({ type: "uuid" });
    uuidType.addReference(python_exports.reference({ name: "UUID", modulePath: ["uuid"] }));
    return uuidType;
  }
  static optional(value) {
    const optionalType = new this({ type: "optional", value });
    optionalType.addReference(python_exports.reference({ name: "Optional", modulePath: ["typing"] }));
    optionalType.inheritReferences(value);
    return optionalType;
  }
  static union(values) {
    const unionType = new this({ type: "union", values });
    unionType.addReference(python_exports.reference({ name: "Union", modulePath: ["typing"] }));
    values.forEach((value) => unionType.inheritReferences(value));
    return unionType;
  }
  static any() {
    const anyType = new this({ type: "any" });
    anyType.addReference(python_exports.reference({ name: "Any", modulePath: ["typing"] }));
    return anyType;
  }
  static datetime() {
    const datetimeType = new this({ type: "datetime" });
    datetimeType.addReference(python_exports.reference({ name: "datetime", modulePath: ["datetime"] }));
    return datetimeType;
  }
  static reference(value) {
    const referenceType = new this({ type: "reference", value });
    referenceType.addReference(value);
    return referenceType;
  }
  static literal(value) {
    const literalType = new this({ type: "literal", value });
    literalType.addReference(python_exports.reference({ name: "Literal", modulePath: ["typing"] }));
    return literalType;
  }
  write(writer) {
    switch (this.internalType.type) {
      case "int":
        writer.write("int");
        break;
      case "float":
        writer.write("float");
        break;
      case "bool":
        writer.write("bool");
        break;
      case "str":
        writer.write("str");
        break;
      case "bytes":
        writer.write("bytes");
        break;
      case "list":
        writer.write("List[");
        this.internalType.value.write(writer);
        writer.write("]");
        break;
      case "set":
        writer.write("Set[");
        this.internalType.value.write(writer);
        writer.write("]");
        break;
      case "tuple":
        writer.write("Tuple[");
        this.internalType.values.forEach((value, index) => {
          if (index > 0) {
            writer.write(", ");
          }
          value.write(writer);
        });
        writer.write("]");
        break;
      case "dict":
        writer.write("Dict[");
        this.internalType.keyType.write(writer);
        writer.write(", ");
        this.internalType.valueType.write(writer);
        writer.write("]");
        break;
      case "none":
        writer.write("None");
        break;
      case "uuid":
        writer.write("UUID");
        break;
      case "optional":
        writer.write("Optional[");
        this.internalType.value.write(writer);
        writer.write("]");
        break;
      case "union":
        writer.write("Union[");
        this.internalType.values.forEach((value, index) => {
          if (index > 0) {
            writer.write(", ");
          }
          value.write(writer);
        });
        writer.write("]");
        break;
      case "any":
        writer.write("Any");
        break;
      case "reference":
        this.internalType.value.write(writer);
        break;
      case "datetime":
        writer.write("datetime");
        break;
      case "literal":
        if (typeof this.internalType.value === "string") {
          writer.write(`Literal["${this.internalType.value}"]`);
        } else {
          writer.write(`Literal[${this.internalType.value}]`);
        }
        break;
      default:
        (0, import_core_utils3.assertNever)(this.internalType);
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Type
});
